/****************************************************************/
/*                                                              */
/*          Advanced Navigation Packet Protocol Library         */
/*       C Language Static Air Data Unit SDK, Version 1.0       */
/*             Copyright 2016, Advanced Navigation              */
/*                                                              */
/****************************************************************/
/*
 * Copyright (C) 2016 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>

#include "an_packet_protocol.h"
#include "air_data_unit_data_dump.h"
#include "air_data_unit_packets.h"


void an_packet_transmit(an_packet_t *an_packet)
{
	an_packet_encode(an_packet);

	/* it is up to the user to define the data transmit function which should have
	 * the prototype: send_function(uint8_t *data, int length); example usage below
	 *
	 * serial_port_transmit(an_packet_pointer(an_packet), an_packet_size(an_packet));
	 */
}

int main()
{
	an_decoder_t an_decoder;
	an_packet_t an_packet;
	int data_count = 0;
	int bytes_to_copy;
	
	air_data_packet_t air_data_packet;
	
	an_decoder_initialise(&an_decoder);
	
	/* iterate through an example data dump from OBDII Odometer */
	while(data_count < sizeof(air_data_unit_data_dump))
	{
		/* calculate the number of bytes to copy */
		bytes_to_copy = an_decoder_size(&an_decoder);
		if(bytes_to_copy > sizeof(air_data_unit_data_dump) - data_count) bytes_to_copy = sizeof(air_data_unit_data_dump) - data_count;
		
		/* fill the decode buffer with the data */
		memcpy(an_decoder_pointer(&an_decoder), &air_data_unit_data_dump[data_count], bytes_to_copy*sizeof(uint8_t));
		an_decoder_increment(&an_decoder, bytes_to_copy);
		
		/* increase the iterator by the number of bytes copied */
		data_count += bytes_to_copy;
		
		/* decode all packets in the internal buffer */
		while(an_packet_decode(&an_decoder, &an_packet))
		{
			if(an_packet.id == packet_id_air_data) /* odometer packet */
			{
				/* copy all the binary data into the typedef struct for the packet */
				/* this allows easy access to all the different values             */
				if(decode_air_data_packet(&air_data_packet, &an_packet) == 0)
				{
					printf("Air Data Packet:\n");
					printf("\tAltitude = %f m, Airspeed = %f m/s, Altitude Valid = %d, Airspeed Valid = %d\n", air_data_packet.altitude, air_data_packet.airspeed, air_data_packet.status.b.altitude_valid, air_data_packet.status.b.airspeed_valid);
				}
			}
			else
			{		
				printf("Packet ID %u of Length %u\n", an_packet.id, an_packet.length);
			}
		}
	}
	
	return EXIT_SUCCESS;
}
