/****************************************************************/
/*                                                              */
/*      Advanced Navigation Packet Protocol Library             */
/*            Java Language SDK, Version 1.0                    */
/*             Copyright 2016, Advanced Navigation              */
/*                                                              */
/****************************************************************/
/*
 * Copyright (C) 2016 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.airDataUnitViewer;

import jssc.*;

import java.awt.EventQueue;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.DefaultComboBoxModel;

import com.advancednavigation.anPackets.*;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.JPanel;
import javax.swing.border.TitledBorder;
import javax.swing.plaf.metal.MetalBorders.TextFieldBorder;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JCheckBox;

public class Main
{
	private JFrame frmAirDataUnitViewer;
	private JFrame frmAbout = null;
	private JComboBox<String> comboBoxPort;
	private JComboBox<String> comboBoxBaud;
	private JButton btnConnect;

	// Serial Port
	private SerialPort serialPort;
	protected Boolean serialConnected = false;
	private Boolean serialPortsAvailable = false;
	private JPanel panel;
	private JLabel lblAirspeed;
	private JLabel lblRate;
	protected JTextField textFieldAirspeed;
	protected JTextField textFieldRate;
	private JLabel lblHz;
	int packetCount = 0;
	private JMenuBar menuBar;
	private JMenu mnFile;
	private JMenu mnHelp;
	private JMenuItem mntmAbout;
	private JMenuItem mntmExit;
	private JLabel lblDelay;
	private JTextField textFieldAirspeedDelay;
	private JLabel lblSec;
	private JComboBox<String> comboBoxSpeedUnits;
	private JComboBox<String> comboBoxDistanceUnits;
	private JLabel lblAltitudeDelay;
	private JTextField textFieldAltitudeDelay;
	private JLabel lblS;
	private JLabel lblAltitude;
	private JTextField textFieldAltitude;
	private JLabel lblAltitudeStandardDeviation;
	private JTextField textFieldAltitudeStandardDeviation;
	private JLabel lblM;
	private JLabel lblAirspeedStandardDeviation;
	private JTextField textFieldAirspeedStandardDeviation;
	private JLabel lblM_1;
	private JLabel lblFlags;
	private JTextField textFieldFlags;
	private JLabel lblAbsolutePressure;
	private JLabel lblDifferentialPressure;
	private JTextField textAbsolutePressure;
	private JTextField textDifferentialPressure;
	private JLabel lblPa;
	private JLabel lblPa_1;

	/*
	 * Code for sending packets
	 * 
	 * ANPacket packet = new ANPacket(1, ANPacket.PACKET_ID_REQUEST); packet.data[0] = ANPacket.PACKET_ID_BOOT_MODE; serialOutput.write(ANPacketDecoder.packetEncode(packet));
	 * 
	 * The above example sends a packet of length 1 requesting the device boot mode packet The serial port must be connected for the above code to work, you can use the boolean serialConnected to check this
	 */

	/**
	 * Launch the application.
	 */
	public static void main(String[] args)
	{
		EventQueue.invokeLater(new Runnable()
		{
			public void run()
			{
				try
				{
					Main window = new Main();
					window.frmAirDataUnitViewer.setVisible(true);
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public Main()
	{
		initialize();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize()
	{
		// this sets swing to use the native look and feel, much more
		// aesthetically pleasing
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch (Exception e)
		{
		}

		// all of the swing components and layout management is best edited
		// using the windowbuilder GUI
		frmAirDataUnitViewer = new JFrame();
		frmAirDataUnitViewer.setTitle("Air Data Unit Viewer");
		frmAirDataUnitViewer.setBounds(100, 100, 650, 600);
		frmAirDataUnitViewer.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[] { 10, 0, 0, 90, 5, 0 };
		gridBagLayout.rowHeights = new int[] { 10, 0, 0, 5, 0 };
		gridBagLayout.columnWeights = new double[] { 0.0, 1.0, 1.0, 0.0, 0.0, Double.MIN_VALUE };
		gridBagLayout.rowWeights = new double[] { 0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE };
		frmAirDataUnitViewer.getContentPane().setLayout(gridBagLayout);
		try
		{
			frmAirDataUnitViewer.setIconImage(ImageIO.read(Main.class.getResourceAsStream("/com/advancednavigation/images/an_icon.png")));
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		comboBoxPort = new JComboBox<String>();
		comboBoxPort.setEnabled(false);
		comboBoxPort.setModel(new DefaultComboBoxModel<String>(new String[] { "No Serial Ports" }));
		GridBagConstraints gbc_comboBoxPort = new GridBagConstraints();
		gbc_comboBoxPort.insets = new Insets(0, 0, 5, 5);
		gbc_comboBoxPort.fill = GridBagConstraints.HORIZONTAL;
		gbc_comboBoxPort.gridx = 1;
		gbc_comboBoxPort.gridy = 1;
		frmAirDataUnitViewer.getContentPane().add(comboBoxPort, gbc_comboBoxPort);

		comboBoxBaud = new JComboBox<String>();
		comboBoxBaud.setModel(new DefaultComboBoxModel<String>(new String[] { "1200", "2400", "4800", "9600", "19200", "38400", "57600", "115200", "230400", "460800", "500000", "921600", "1000000" }));
		comboBoxBaud.setSelectedIndex(7);
		GridBagConstraints gbc_comboBoxBaud = new GridBagConstraints();
		gbc_comboBoxBaud.insets = new Insets(0, 0, 5, 5);
		gbc_comboBoxBaud.fill = GridBagConstraints.HORIZONTAL;
		gbc_comboBoxBaud.gridx = 2;
		gbc_comboBoxBaud.gridy = 1;
		frmAirDataUnitViewer.getContentPane().add(comboBoxBaud, gbc_comboBoxBaud);

		btnConnect = new JButton("Connect");
		btnConnect.setEnabled(false);
		btnConnect.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				// if the serial port is not connected, this is a connect action
				// if the serial port is connected, this is a disconnect action
				if (!serialConnected)
				{
					try
					{
						serialPort = new SerialPort((String) comboBoxPort.getSelectedItem());
						serialPort.openPort();
						serialPort.setParams(Integer.parseInt((String) comboBoxBaud.getSelectedItem()), SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
						serialPort.setEventsMask(SerialPort.MASK_RXCHAR);

						// add an event listener, which is the subclass at the
						// bottom of this file
						serialPort.addEventListener(new SerialReader());

						// change the button to be disconnect once we are
						// connected
						btnConnect.setText("Disconnect");
						serialConnected = true;
						comboBoxPort.setEnabled(false);
						comboBoxBaud.setEnabled(false);
					}
					catch (Exception exception)
					{
						System.err.println(exception.toString());
						serialPortClose();
					}
				}
				else
				{
					serialPortClose();
				}
			}
		});
		GridBagConstraints gbc_btnConnect = new GridBagConstraints();
		gbc_btnConnect.fill = GridBagConstraints.HORIZONTAL;
		gbc_btnConnect.insets = new Insets(0, 0, 5, 5);
		gbc_btnConnect.gridx = 3;
		gbc_btnConnect.gridy = 1;
		frmAirDataUnitViewer.getContentPane().add(btnConnect, gbc_btnConnect);

		panel = new JPanel();
		panel.setBorder(new TitledBorder(null, "Data", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		GridBagConstraints gbc_panel = new GridBagConstraints();
		gbc_panel.gridwidth = 3;
		gbc_panel.insets = new Insets(5, 5, 5, 5);
		gbc_panel.fill = GridBagConstraints.BOTH;
		gbc_panel.gridx = 1;
		gbc_panel.gridy = 2;
		frmAirDataUnitViewer.getContentPane().add(panel, gbc_panel);
		GridBagLayout gbl_panel = new GridBagLayout();
		gbl_panel.columnWidths = new int[] { 0, 0, 5, 0 };
		gbl_panel.rowHeights = new int[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		gbl_panel.columnWeights = new double[] { 0.0, 1.0, 1.0, Double.MIN_VALUE };
		gbl_panel.rowWeights = new double[] { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE };
		panel.setLayout(gbl_panel);
		
		lblAbsolutePressure = new JLabel("Absolute Pressure:");
		GridBagConstraints gbc_lblAbsolutePressure = new GridBagConstraints();
		gbc_lblAbsolutePressure.anchor = GridBagConstraints.EAST;
		gbc_lblAbsolutePressure.insets = new Insets(5, 5, 5, 5);
		gbc_lblAbsolutePressure.gridx = 0;
		gbc_lblAbsolutePressure.gridy = 0;
		panel.add(lblAbsolutePressure, gbc_lblAbsolutePressure);
		
		textAbsolutePressure = new JTextField();
		textAbsolutePressure.setEditable(false);
		GridBagConstraints gbc_textAbsolutePressure = new GridBagConstraints();
		gbc_textAbsolutePressure.insets = new Insets(5, 5, 5, 5);
		gbc_textAbsolutePressure.fill = GridBagConstraints.HORIZONTAL;
		gbc_textAbsolutePressure.gridx = 1;
		gbc_textAbsolutePressure.gridy = 0;
		panel.add(textAbsolutePressure, gbc_textAbsolutePressure);
		textAbsolutePressure.setColumns(10);
		
		lblPa = new JLabel("Pa");
		GridBagConstraints gbc_lblPa = new GridBagConstraints();
		gbc_lblPa.anchor = GridBagConstraints.WEST;
		gbc_lblPa.insets = new Insets(5, 5, 5, 5);
		gbc_lblPa.gridx = 2;
		gbc_lblPa.gridy = 0;
		panel.add(lblPa, gbc_lblPa);
		
		lblDifferentialPressure = new JLabel("Differential Pressure:");
		GridBagConstraints gbc_lblDifferentialPressure = new GridBagConstraints();
		gbc_lblDifferentialPressure.insets = new Insets(5, 5, 5, 5);
		gbc_lblDifferentialPressure.anchor = GridBagConstraints.EAST;
		gbc_lblDifferentialPressure.gridx = 0;
		gbc_lblDifferentialPressure.gridy = 1;
		panel.add(lblDifferentialPressure, gbc_lblDifferentialPressure);
		
		textDifferentialPressure = new JTextField();
		textDifferentialPressure.setEditable(false);
		GridBagConstraints gbc_textDifferentialPressure = new GridBagConstraints();
		gbc_textDifferentialPressure.insets = new Insets(5, 5, 5, 5);
		gbc_textDifferentialPressure.fill = GridBagConstraints.HORIZONTAL;
		gbc_textDifferentialPressure.gridx = 1;
		gbc_textDifferentialPressure.gridy = 1;
		panel.add(textDifferentialPressure, gbc_textDifferentialPressure);
		textDifferentialPressure.setColumns(10);
		
		lblPa_1 = new JLabel("Pa");
		GridBagConstraints gbc_lblPa_1 = new GridBagConstraints();
		gbc_lblPa_1.anchor = GridBagConstraints.WEST;
		gbc_lblPa_1.insets = new Insets(5, 5, 5, 5);
		gbc_lblPa_1.gridx = 2;
		gbc_lblPa_1.gridy = 1;
		panel.add(lblPa_1, gbc_lblPa_1);

		lblAltitudeDelay = new JLabel("Altitude Delay:");
		GridBagConstraints gbc_lblAltitudeDelay = new GridBagConstraints();
		gbc_lblAltitudeDelay.anchor = GridBagConstraints.EAST;
		gbc_lblAltitudeDelay.insets = new Insets(5, 5, 5, 5);
		gbc_lblAltitudeDelay.gridx = 0;
		gbc_lblAltitudeDelay.gridy = 2;
		panel.add(lblAltitudeDelay, gbc_lblAltitudeDelay);

		textFieldAltitudeDelay = new JTextField();
		textFieldAltitudeDelay.setEditable(false);
		GridBagConstraints gbc_textFieldAltitudeDelay = new GridBagConstraints();
		gbc_textFieldAltitudeDelay.insets = new Insets(5, 5, 5, 5);
		gbc_textFieldAltitudeDelay.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldAltitudeDelay.gridx = 1;
		gbc_textFieldAltitudeDelay.gridy = 2;
		panel.add(textFieldAltitudeDelay, gbc_textFieldAltitudeDelay);
		textFieldAltitudeDelay.setColumns(10);

		lblS = new JLabel("s");
		GridBagConstraints gbc_lblS = new GridBagConstraints();
		gbc_lblS.anchor = GridBagConstraints.WEST;
		gbc_lblS.insets = new Insets(5, 5, 5, 0);
		gbc_lblS.gridx = 2;
		gbc_lblS.gridy = 2;
		panel.add(lblS, gbc_lblS);

		lblDelay = new JLabel("Airspeed Delay:");
		GridBagConstraints gbc_lblDelay = new GridBagConstraints();
		gbc_lblDelay.anchor = GridBagConstraints.EAST;
		gbc_lblDelay.insets = new Insets(5, 5, 5, 5);
		gbc_lblDelay.gridx = 0;
		gbc_lblDelay.gridy = 3;
		panel.add(lblDelay, gbc_lblDelay);

		textFieldAirspeedDelay = new JTextField();
		textFieldAirspeedDelay.setEditable(false);
		GridBagConstraints gbc_textFieldAirspeedDelay = new GridBagConstraints();
		gbc_textFieldAirspeedDelay.insets = new Insets(5, 5, 5, 5);
		gbc_textFieldAirspeedDelay.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldAirspeedDelay.gridx = 1;
		gbc_textFieldAirspeedDelay.gridy = 3;
		panel.add(textFieldAirspeedDelay, gbc_textFieldAirspeedDelay);
		textFieldAirspeedDelay.setColumns(10);

		lblSec = new JLabel("s");
		GridBagConstraints gbc_lblSec = new GridBagConstraints();
		gbc_lblSec.anchor = GridBagConstraints.WEST;
		gbc_lblSec.insets = new Insets(5, 5, 5, 0);
		gbc_lblSec.gridx = 2;
		gbc_lblSec.gridy = 3;
		panel.add(lblSec, gbc_lblSec);

		lblAltitude = new JLabel("Altitude:");
		GridBagConstraints gbc_lblAltitude = new GridBagConstraints();
		gbc_lblAltitude.anchor = GridBagConstraints.EAST;
		gbc_lblAltitude.insets = new Insets(5, 5, 5, 5);
		gbc_lblAltitude.gridx = 0;
		gbc_lblAltitude.gridy = 4;
		panel.add(lblAltitude, gbc_lblAltitude);

		textFieldAltitude = new JTextField();
		textFieldAltitude.setEditable(false);
		GridBagConstraints gbc_textFieldAltitude = new GridBagConstraints();
		gbc_textFieldAltitude.insets = new Insets(5, 5, 5, 5);
		gbc_textFieldAltitude.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldAltitude.gridx = 1;
		gbc_textFieldAltitude.gridy = 4;
		panel.add(textFieldAltitude, gbc_textFieldAltitude);
		textFieldAltitude.setColumns(10);

		comboBoxDistanceUnits = new JComboBox<String>();
		comboBoxDistanceUnits.setMaximumRowCount(3);
		comboBoxDistanceUnits.setModel(new DefaultComboBoxModel<String>(new String[] { "ft", "m", "km", "mi" }));
		comboBoxDistanceUnits.setSelectedIndex(0);
		GridBagConstraints gbc_comboBoxDistanceUnits = new GridBagConstraints();
		gbc_comboBoxDistanceUnits.insets = new Insets(5, 5, 5, 0);
		gbc_comboBoxDistanceUnits.fill = GridBagConstraints.HORIZONTAL;
		gbc_comboBoxDistanceUnits.gridx = 2;
		gbc_comboBoxDistanceUnits.gridy = 4;
		panel.add(comboBoxDistanceUnits, gbc_comboBoxDistanceUnits);

		lblAirspeed = new JLabel("Airspeed:");
		GridBagConstraints gbc_lblAirspeed = new GridBagConstraints();
		gbc_lblAirspeed.anchor = GridBagConstraints.EAST;
		gbc_lblAirspeed.insets = new Insets(5, 5, 5, 5);
		gbc_lblAirspeed.gridx = 0;
		gbc_lblAirspeed.gridy = 5;
		panel.add(lblAirspeed, gbc_lblAirspeed);

		textFieldAirspeed = new JTextField();
		textFieldAirspeed.setEditable(false);
		GridBagConstraints gbc_textFieldAirspeed = new GridBagConstraints();
		gbc_textFieldAirspeed.insets = new Insets(5, 5, 5, 5);
		gbc_textFieldAirspeed.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldAirspeed.gridx = 1;
		gbc_textFieldAirspeed.gridy = 5;
		panel.add(textFieldAirspeed, gbc_textFieldAirspeed);
		textFieldAirspeed.setColumns(10);

		comboBoxSpeedUnits = new JComboBox<String>();
		comboBoxSpeedUnits.setMaximumRowCount(3);
		comboBoxSpeedUnits.setModel(new DefaultComboBoxModel<String>(new String[] { "knots", "km/h", "mph", "m/s" }));
		comboBoxSpeedUnits.setSelectedIndex(0);
		GridBagConstraints gbc_comboBoxSpeedUnits = new GridBagConstraints();
		gbc_comboBoxSpeedUnits.insets = new Insets(5, 5, 5, 0);
		gbc_comboBoxSpeedUnits.fill = GridBagConstraints.HORIZONTAL;
		gbc_comboBoxSpeedUnits.gridx = 2;
		gbc_comboBoxSpeedUnits.gridy = 5;
		panel.add(comboBoxSpeedUnits, gbc_comboBoxSpeedUnits);

		lblAltitudeStandardDeviation = new JLabel("Altitude Standard Deviation:");
		GridBagConstraints gbc_lblAltitudeStandardDeviation = new GridBagConstraints();
		gbc_lblAltitudeStandardDeviation.anchor = GridBagConstraints.EAST;
		gbc_lblAltitudeStandardDeviation.insets = new Insets(5, 5, 5, 5);
		gbc_lblAltitudeStandardDeviation.gridx = 0;
		gbc_lblAltitudeStandardDeviation.gridy = 6;
		panel.add(lblAltitudeStandardDeviation, gbc_lblAltitudeStandardDeviation);

		textFieldAltitudeStandardDeviation = new JTextField();
		textFieldAltitudeStandardDeviation.setEditable(false);
		GridBagConstraints gbc_textFieldAltitudeStandardDeviation = new GridBagConstraints();
		gbc_textFieldAltitudeStandardDeviation.insets = new Insets(5, 5, 5, 5);
		gbc_textFieldAltitudeStandardDeviation.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldAltitudeStandardDeviation.gridx = 1;
		gbc_textFieldAltitudeStandardDeviation.gridy = 6;
		panel.add(textFieldAltitudeStandardDeviation, gbc_textFieldAltitudeStandardDeviation);
		textFieldAltitudeStandardDeviation.setColumns(10);

		lblM = new JLabel("m");
		GridBagConstraints gbc_lblM = new GridBagConstraints();
		gbc_lblM.anchor = GridBagConstraints.WEST;
		gbc_lblM.insets = new Insets(5, 5, 5, 0);
		gbc_lblM.gridx = 2;
		gbc_lblM.gridy = 6;
		panel.add(lblM, gbc_lblM);

		lblAirspeedStandardDeviation = new JLabel("Airspeed Standard Deviation:");
		GridBagConstraints gbc_lblAirspeedStandardDeviation = new GridBagConstraints();
		gbc_lblAirspeedStandardDeviation.anchor = GridBagConstraints.EAST;
		gbc_lblAirspeedStandardDeviation.insets = new Insets(5, 5, 5, 5);
		gbc_lblAirspeedStandardDeviation.gridx = 0;
		gbc_lblAirspeedStandardDeviation.gridy = 7;
		panel.add(lblAirspeedStandardDeviation, gbc_lblAirspeedStandardDeviation);

		textFieldAirspeedStandardDeviation = new JTextField();
		textFieldAirspeedStandardDeviation.setEditable(false);
		GridBagConstraints gbc_textFieldAirspeedStandardDeviation = new GridBagConstraints();
		gbc_textFieldAirspeedStandardDeviation.insets = new Insets(5, 5, 5, 5);
		gbc_textFieldAirspeedStandardDeviation.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldAirspeedStandardDeviation.gridx = 1;
		gbc_textFieldAirspeedStandardDeviation.gridy = 7;
		panel.add(textFieldAirspeedStandardDeviation, gbc_textFieldAirspeedStandardDeviation);
		textFieldAirspeedStandardDeviation.setColumns(10);

		lblM_1 = new JLabel("m/s");
		GridBagConstraints gbc_lblM_1 = new GridBagConstraints();
		gbc_lblM_1.anchor = GridBagConstraints.WEST;
		gbc_lblM_1.insets = new Insets(5, 5, 5, 0);
		gbc_lblM_1.gridx = 2;
		gbc_lblM_1.gridy = 7;
		panel.add(lblM_1, gbc_lblM_1);

		lblFlags = new JLabel("Flags:");
		GridBagConstraints gbc_lblFlags = new GridBagConstraints();
		gbc_lblFlags.anchor = GridBagConstraints.EAST;
		gbc_lblFlags.insets = new Insets(5, 5, 5, 5);
		gbc_lblFlags.gridx = 0;
		gbc_lblFlags.gridy = 8;
		panel.add(lblFlags, gbc_lblFlags);

		textFieldFlags = new JTextField();
		textFieldFlags.setEditable(false);
		GridBagConstraints gbc_textFieldFlags = new GridBagConstraints();
		gbc_textFieldFlags.insets = new Insets(5, 5, 5, 5);
		gbc_textFieldFlags.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldFlags.gridx = 1;
		gbc_textFieldFlags.gridy = 8;
		panel.add(textFieldFlags, gbc_textFieldFlags);
		textFieldFlags.setColumns(10);

		lblRate = new JLabel("Rate:");
		GridBagConstraints gbc_lblRate = new GridBagConstraints();
		gbc_lblRate.anchor = GridBagConstraints.EAST;
		gbc_lblRate.insets = new Insets(5, 5, 0, 5);
		gbc_lblRate.gridx = 0;
		gbc_lblRate.gridy = 9;
		panel.add(lblRate, gbc_lblRate);

		textFieldRate = new JTextField();
		textFieldRate.setEditable(false);
		GridBagConstraints gbc_textFieldRate = new GridBagConstraints();
		gbc_textFieldRate.insets = new Insets(5, 5, 0, 5);
		gbc_textFieldRate.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldRate.gridx = 1;
		gbc_textFieldRate.gridy = 9;
		panel.add(textFieldRate, gbc_textFieldRate);
		textFieldRate.setColumns(10);

		lblHz = new JLabel("Hz");
		GridBagConstraints gbc_lblHz = new GridBagConstraints();
		gbc_lblHz.anchor = GridBagConstraints.WEST;
		gbc_lblHz.insets = new Insets(5, 5, 0, 0);
		gbc_lblHz.gridx = 2;
		gbc_lblHz.gridy = 9;
		panel.add(lblHz, gbc_lblHz);

		menuBar = new JMenuBar();
		frmAirDataUnitViewer.setJMenuBar(menuBar);

		mnFile = new JMenu("File");
		menuBar.add(mnFile);

		mntmExit = new JMenuItem("Exit");
		mntmExit.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				frmAirDataUnitViewer.dispose();
				System.exit(0);
			}
		});
		mnFile.add(mntmExit);

		mnHelp = new JMenu("Help");
		menuBar.add(mnHelp);

		mntmAbout = new JMenuItem("About");
		mntmAbout.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				if (frmAbout == null)
				{
					frmAbout = new JFrame();
					frmAbout.setTitle("About");
					frmAbout.setBounds(100, 100, 350, 250);
					frmAbout.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
					GridBagLayout gridBagLayout = new GridBagLayout();
					gridBagLayout.columnWidths = new int[] { 0, 0, 0, 0 };
					gridBagLayout.rowHeights = new int[] { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
					gridBagLayout.columnWeights = new double[] { 1.0, 0.0, 1.0, Double.MIN_VALUE };
					gridBagLayout.rowWeights = new double[] { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, Double.MIN_VALUE };
					frmAbout.getContentPane().setLayout(gridBagLayout);

					JLabel lblNewLabel = new JLabel("");
					lblNewLabel.setIcon(new ImageIcon(Main.class.getResource("/com/advancednavigation/images/an_logo.png")));
					GridBagConstraints gbc_lblNewLabel = new GridBagConstraints();
					gbc_lblNewLabel.insets = new Insets(0, 0, 10, 5);
					gbc_lblNewLabel.gridx = 1;
					gbc_lblNewLabel.gridy = 1;
					frmAbout.getContentPane().add(lblNewLabel, gbc_lblNewLabel);

					JLabel lblObdii = new JLabel("Air Data Unit Viewer Version 1.1");
					GridBagConstraints gbc_lblObdii = new GridBagConstraints();
					gbc_lblObdii.insets = new Insets(0, 0, 5, 5);
					gbc_lblObdii.gridx = 1;
					gbc_lblObdii.gridy = 2;
					frmAbout.getContentPane().add(lblObdii, gbc_lblObdii);

					Date date;
					try
					{
						date = new Date(Main.class.getResource("Main.class").openConnection().getLastModified());
					}
					catch (Exception exception)
					{
						date = new GregorianCalendar(2013, 0, 0).getTime();
					}
					JLabel lblBuildDate = new JLabel("Build Date: " + date);
					GridBagConstraints gbc_lblBuildDate = new GridBagConstraints();
					gbc_lblBuildDate.insets = new Insets(0, 0, 5, 5);
					gbc_lblBuildDate.gridx = 1;
					gbc_lblBuildDate.gridy = 3;
					frmAbout.getContentPane().add(lblBuildDate, gbc_lblBuildDate);

					JLabel lblCopyrightc = new JLabel("Copyright (c) 2016");
					GridBagConstraints gbc_lblCopyrightc = new GridBagConstraints();
					gbc_lblCopyrightc.insets = new Insets(0, 0, 5, 5);
					gbc_lblCopyrightc.gridx = 1;
					gbc_lblCopyrightc.gridy = 4;
					frmAbout.getContentPane().add(lblCopyrightc, gbc_lblCopyrightc);

					JLabel lblAdvancedNavigationPty = new JLabel("Advanced Navigation");
					GridBagConstraints gbc_lblAdvancedNavigationPty = new GridBagConstraints();
					gbc_lblAdvancedNavigationPty.insets = new Insets(0, 0, 5, 5);
					gbc_lblAdvancedNavigationPty.gridx = 1;
					gbc_lblAdvancedNavigationPty.gridy = 5;
					frmAbout.getContentPane().add(lblAdvancedNavigationPty, gbc_lblAdvancedNavigationPty);

					JLabel lblHttpwwwadvancednavigationcomau = new JLabel("http://www.advancednavigation.com.au");
					GridBagConstraints gbc_lblHttpwwwadvancednavigationcomau = new GridBagConstraints();
					gbc_lblHttpwwwadvancednavigationcomau.insets = new Insets(0, 0, 5, 5);
					gbc_lblHttpwwwadvancednavigationcomau.gridx = 1;
					gbc_lblHttpwwwadvancednavigationcomau.gridy = 6;
					frmAbout.getContentPane().add(lblHttpwwwadvancednavigationcomau, gbc_lblHttpwwwadvancednavigationcomau);
					frmAbout.setVisible(true);
				}
				else
				{
					frmAbout.setVisible(true);
					frmAbout.toFront();
				}
			}
		});
		mnHelp.add(mntmAbout);

		// create a thread to update the available serials ports once a second
		// it is done in a new thread because under windows it can block for up
		// to 10 seconds
		Thread portScannerThread = new Thread(new PortScanner());
		portScannerThread.start();
	}

	public class PortScanner implements Runnable
	{
		public void run()
		{
			while (true)
			{
				if (!serialConnected)
				{
					SwingUtilities.invokeLater(new Runnable()
					{
						public void run()
						{
							if (!serialConnected)
							{
								String[] portNames = SerialPortList.getPortNames();
								int comboLength = comboBoxPort.getItemCount();
								if (!serialPortsAvailable) comboLength = 0;
								Boolean portsChanged = portNames.length != comboLength;
								if (!portsChanged)
								{
									for (int i = 0; i < portNames.length; i++)
									{
										if (portNames[i].compareTo((String) comboBoxPort.getItemAt(i)) != 0)
										{
											portsChanged = true;
											break;
										}
									}
								}
								if (portsChanged)
								{
									comboBoxPort.removeAllItems();
									for (int i = 0; i < portNames.length; i++)
									{
										comboBoxPort.addItem(portNames[i]);
									}
									if (portNames.length == 0)
									{
										comboBoxPort.addItem("No Serial Ports");
										comboBoxPort.setSelectedIndex(0);
										comboBoxPort.setEnabled(false);
										btnConnect.setEnabled(false);
										serialPortsAvailable = false;
									}
									else
									{
										comboBoxPort.setSelectedIndex(0);
										comboBoxPort.setEnabled(true);
										btnConnect.setEnabled(true);
										serialPortsAvailable = true;
									}
								}
							}
						}
					});
				}
				else
				{
					textFieldRate.setText(Integer.toString(packetCount));
					if (packetCount == 0)
					{
						textFieldAltitudeDelay.setText("");
						textFieldAirspeedDelay.setText("");
						textFieldAltitude.setText("");
						textFieldAirspeed.setText("");
						textFieldAirspeedStandardDeviation.setText("");
						textFieldAltitudeStandardDeviation.setText("");
						textFieldFlags.setText("");
					}
					packetCount = 0;
				}
				try
				{
					Thread.sleep(1000);
				}
				catch (InterruptedException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	private void serialPortClose()
	{
		try
		{
			serialPort.removeEventListener();
			serialPort.closePort();
		}
		catch (SerialPortException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		btnConnect.setText("Connect");
		serialConnected = false;
		comboBoxPort.setEnabled(true);
		comboBoxBaud.setEnabled(true);
	}

	/*
	 * This class is the event listener for received serial data Whenever serial data is received the serialEvent is called which adds the received data to the decoders buffer and then checks whether any packets can be decoded.
	 */
	public class SerialReader implements SerialPortEventListener
	{
		ANPacketDecoder packetDecoder;

		public SerialReader()
		{
			packetDecoder = new ANPacketDecoder();
		}

		public void serialEvent(SerialPortEvent event)
		{
			if (event.isRXCHAR())
			{
				try
				{
					byte[] buffer = serialPort.readBytes();
					if (buffer != null)
					{
						for (int i = 0; i < buffer.length; i++)
						{
							if (packetDecoder.bufferLength < packetDecoder.buffer.length)
							{
								packetDecoder.buffer[packetDecoder.bufferLength++] = buffer[i];
							}
						}

						ANPacket packet = null;
						while ((packet = packetDecoder.packetDecode()) != null)
						{
							switch (packet.id)
							{
								case ANPacket.PACKET_ID_RAW_SENSORS:
									if(packet.length == 9)
									{
										ANPacket28 anPacket28 = new ANPacket28(packet);
										textAbsolutePressure.setText(Float.toString(anPacket28.absolutePressure));
										textDifferentialPressure.setText(Float.toString(anPacket28.differentialPressure));
									}
									break;
								case ANPacket.PACKET_ID_AIR_DATA:
									if (packet.length == 25)
									{
										final ANPacket68 anPacket68 = new ANPacket68(packet);
										packetCount++;

										SwingUtilities.invokeLater(new Runnable()
										{
											public void run()
											{
												double altitude = anPacket68.barometricAltitude;
												double airspeed = anPacket68.airspeed;
												switch (comboBoxSpeedUnits.getSelectedIndex())
												{
													case 0:
														airspeed *= 1.94384449; // knots
														break;
													case 1:
														airspeed *= 3.6; // kilometres/hour
														break;
													case 2:
														airspeed *= 2.23693629; // miles/hour
														break;
													default: // default (index 3) is metres/second
														break;
												}
												switch (comboBoxDistanceUnits.getSelectedIndex())
												{
													case 0:
														altitude *= 3.2808399; // feet
														break;
													case 2:
														altitude *= 0.001; // kilometres
														break;
													case 3:
														altitude *= 0.00062137119; // 
														break;
													default:
														break;
												}
												textFieldAltitudeDelay.setText(String.format("%.3f", anPacket68.barometricAltitudeDelay));
												textFieldAirspeedDelay.setText(String.format("%.3f", anPacket68.airSpeedDelay));
												textFieldAirspeed.setText(String.format("%.3f", airspeed));
												textFieldAltitude.setText(String.format("%.3f", altitude));
												textFieldAirspeedStandardDeviation.setText(String.format("%.3f", anPacket68.barometricAltitudeStandardDeviation));
												textFieldAltitudeStandardDeviation.setText(String.format("%.3f", anPacket68.airspeedStandardDeviation));
												textFieldFlags.setText(anPacket68.statusMessageString());
											}
										});
									}
									break;
								default:
									break;
							}
						}
					}
				}
				catch (Exception exception)
				{
					System.err.println(exception.toString());
				}
			}
		}
	}
}
