/****************************************************************/
/*                                                              */
/*      Advanced Navigation Packet Protocol Library             */
/*            Java Language SDK, Version 1.0                    */
/*             Copyright 2016, Advanced Navigation              */
/*                                                              */
/****************************************************************/
/*
 * Copyright (C) 2016 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* Air Data Packet */
public class ANPacket68
{		
	/* seconds */
	public float barometricAltitudeDelay;
	public float airSpeedDelay;
	/* metres */
	public float barometricAltitude;
	/* metres per second */
	public float airspeed;
	/* metres */
	public float barometricAltitudeStandardDeviation;
	/* metres per second */
	public float airspeedStandardDeviation;
	
	public Boolean barometricAltitudeValid;
	public Boolean airspeedValid;
	public Boolean barometricAltitudeOverRange;
	public Boolean airspeedOverRange;
	public Boolean barometricAltitudeSensorFailure;
	public Boolean airspeedSensorFailure;
	
	public ANPacket68(ANPacket packet)
	{
		barometricAltitudeDelay = TypeConversion.bytesToFloat(packet.data, 0);
		airSpeedDelay = TypeConversion.bytesToFloat(packet.data, 4);
		barometricAltitude = TypeConversion.bytesToFloat(packet.data, 8);
		airspeed = TypeConversion.bytesToFloat(packet.data, 12);
		barometricAltitudeStandardDeviation = TypeConversion.bytesToFloat(packet.data, 16);
		airspeedStandardDeviation = TypeConversion.bytesToFloat(packet.data, 20);
		barometricAltitudeValid = (packet.data[24] & 0x01) != 0;
		airspeedValid = (packet.data[24] & 0x02) != 0;
		barometricAltitudeOverRange = (packet.data[24] & 0x04) != 0;
		airspeedOverRange = (packet.data[24] & 0x08) != 0;
		barometricAltitudeSensorFailure = (packet.data[24] & 0x10) != 0;
		airspeedSensorFailure = (packet.data[24] & 0x20) != 0;
	}
	
	public String statusMessageString()
	{
		if(barometricAltitudeSensorFailure)
		{
			return "Barometric Altitude Sensor Failure";
		}
		else if(airspeedSensorFailure)
		{
			return "Airspeed Sensor Failure";
		}
		else if(barometricAltitudeOverRange)
		{
			return "Barometric Altitude Over-range";
		}
		else if(airspeedOverRange)
		{
			return "Airspeed Over-range";
		}
		else if(!barometricAltitudeValid)
		{
			return "Barometric Altitude Invalid";
		}
		else if(!airspeedValid)
		{
			return "Airspeed Invalid";
		}
		else
		{
			return "Healthy";
		}
	}
}
